__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on a "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


if __name__ == '__main__':
    ###### below line CRUCIAL when running Windows, otherwise multiprocessing
    # doesn't work! (not necessary on Linux)
    from findatapy.util import SwimPool;

    SwimPool()

    from findatapy.market import Market, MarketDataRequest, MarketDataGenerator

    from findatapy.util import ConfigManager

    # findatapy has predefined tickers and categories in the conf folder in
    # CSV files which you can modify, that
    # map from tickers/categories to vendor tickers/vendor fields
    #
    # time_series_categories_fields.csv
    # time_series_fields_list.csv
    # time_series_tickers_list.csv
    #
    # You can also add you own files with tickers, like we have done for FX vol
    # like fx_vol_tickers.csv
    #
    # Having these ticker mappings makes it easier to mix and match data from
    # different data sources, and saves us
    # having to remember vendor specific tickers

    cm = ConfigManager().get_instance()

    # Get all the categories for raw data (note this won't include generated
    # categories like fx-vol-market,
    # which aggregate from many other categories)
    categories = list(cm.get_categories_from_tickers())

    print(categories)

    # Filter those categories which include quandl
    quandl_category = [x for x in categories if 'quandl' in x]

    print(quandl_category[0])

    # For this category, get all the tickers and fields which are available
    tickers = cm.get_tickers_list_for_category_str(categories[0])
    fields = cm.get_fields_list_for_category_str(categories[0])

    # We don't need to add the environment (eg. backtest)
    print("For category " + quandl_category[0] + ", tickers = " + str(
        tickers) + ", fields = " + str(fields))

    # Do a more complicated query, get any combinations
    df = cm.free_form_tickers_regex_query(category='fx/*|equities/*',
                                          ret_fields=['category', 'freq'])

    print(df.head(min(5, len(df.index))))

    # Let's search for all the category.data_source.freq.cut.tickers.fields
    # combinations
    # where the category matches the regular expression for which has 'fx' or
    # 'equities' in it and smart group
    df = cm.free_form_tickers_regex_query(category='fx/*|equities/*',
                                          ret_fields=['category',
                                                      'data_source', 'freq',
                                                      'cut', 'tickers',
                                                      'fields'],
                                          smart_group=True)

    print(df.head(min(5, len(df.index))))

    # We'll do the same query again, but this time, we'll convert it into a
    # DataFrame which can use to create
    # a MarketDataRequest
    df_ungrouped = cm.free_form_tickers_regex_query(category='fx/*|equities/*',
                                                    ret_fields=['category',
                                                                'data_source',
                                                                'freq', 'cut',
                                                                'tickers',
                                                                'fields'])

    print(df_ungrouped)

    # We can also do exact matches for multiple queries, where findatapy
    # guesses the what parameters we were intending to find
    # so they don't necessarily have to be in the order
    # category.data_source.freq.cut.ticker etc. They can be in whatever
    # order you want. Note that for this to work, we can not have values common
    # across multiple fields, like tickers and category
    #
    # Here we are searching for the tickers/vendor_tickers combination which
    # matches
    # - fx.daily.quandl (ie. findatapy will guess category='fx', freq='daily'
    # and data_source = 'quandl')
    # - fx.daily.bloomberg (ie. findatapy
    df_guess = cm.free_form_tickers_query(
        ['fx.daily.quandl', 'fx.daily.bloomberg'],
        ret_fields=['tickers', 'vendor_tickers'], list_query=True,
        smart_group=True)

    print(df_guess)

    market = Market(market_data_generator=MarketDataGenerator())

    md_request_list = market.create_md_request_from_dataframe(df_ungrouped)

    print(md_request_list)

    df_tickers = ConfigManager.get_dataframe_tickers()

    print(df_tickers)

    from findatapy.util import DataConstants

    constants = DataConstants()

    # You will likely need to put your own Quandl API key (free from Quandl's
    # website!)
    quandl_api_key = constants.quandl_api_key
    print(market.fetch_market(md_request_str=categories[0],
                              md_request=MarketDataRequest(
                                  quandl_api_key=quandl_api_key)))

    # Or we could create this download of FX Quandl data with a more free form
    # query
    df_quandl_tickers = cm.free_form_tickers_regex_query(category='fx/*',
                                                         data_source='quandl',
                                                         cut='NYC')
    print(market.fetch_market(md_request_df=df_quandl_tickers,
                              md_request=MarketDataRequest(
                                  quandl_api_key=quandl_api_key)))

    # Now lets get all Bloomberg data and Quandl FX data in our predefined
    # tickers for the past week
    # we use ^fx$ as a regular expression, so it will only match with fx, and
    # not any variations like fx-implied-vol
    md_request = MarketDataRequest(start_date='week',
                                   quandl_api_key=quandl_api_key)

    md_request_list = market.create_md_request_from_dataframe(
        cm.free_form_tickers_regex_query(category='^fx$', freq='daily',
                                         data_source='quandl|bloomberg',
                                         cut='NYC'),
        md_request=md_request)

    for m in md_request_list:
        print(market.fetch_market(m))



__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on a "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


if __name__ == '__main__':
    ###### below line CRUCIAL when running Windows, otherwise multiprocessing doesn't work! (not necessary on Linux)
    from findatapy.util import SwimPool;

    SwimPool()

    from findatapy.market import Market, MarketDataRequest, \
        MarketDataGenerator, IOEngine

    market = Market(market_data_generator=MarketDataGenerator())

    # In the config file, we can use keywords 'open', 'high', 'low', 'close'
    # and 'volume' for Yahoo and Google finance data

    # Download equities data from Yahoo
    md_request = MarketDataRequest(
        start_date="decade",  # start date
        data_source='yahoo',  # use Bloomberg as data source
        tickers=['Apple', 'Citigroup'],  # ticker (findatapy)
        fields=['close'],  # which fields to download
        vendor_tickers=['aapl', 'c'],  # ticker (Yahoo)
        vendor_fields=['Close'])  # which Bloomberg fields to download)

    df = market.fetch_market(md_request)

    io = IOEngine()

    # Note: you need to set up Man-AHL's Arctic and MongoDB database for this
    # to work
    # write to Arctic (to MongoDB) - by default use's Arctic's VersionStore
    io.write_time_series_cache_to_disk('stocks', df, engine='arctic',
                                       db_server='127.0.0.1')

    # Read back from Arctic
    df_arctic = io.read_time_series_cache_from_disk('stocks', engine='arctic',
                                                    db_server='127.0.0.1')

    print(df_arctic.tail(n=5))



## test function
if __name__ == "__main__":
    logger = LoggerManager().getLogger(__name__)

    data_constants = DataConstants(
        override_fields={"use_cache_compression": False})

    print(data_constants.use_cache_compression)

    cm = ConfigManager().get_instance()

    categories = cm.get_categories_from_fields()

    logger.info("Categories from fields list")
    print(categories)

    categories = cm.get_categories_from_tickers()

    logger.info("Categories from tickers list")
    print(categories)

    filter = "events"

    categories_filtered = cm.get_categories_from_tickers_selective_filter(
        filter)
    logger.info("Categories from tickers list, filtered by events")
    print(categories_filtered)

    logger.info("For each category, print all tickers and fields")

    for sing in categories:
        split_sing = sing.split(".")
        category = split_sing[0]
        data_source = split_sing[1]
        freq = split_sing[2]
        cut = split_sing[3]

        logger.info("tickers for " + sing)
        tickers = cm.get_tickers_list_for_category(category, data_source, freq,
                                                   cut)

        print(tickers)

        logger.info("fields for " + sing)
        fields = cm.get_fields_list_for_category(category, data_source, freq,
                                                 cut)

        print(fields)

    # test the various converter mechanisms
    output = cm.convert_library_to_vendor_ticker(category="fx",
                                                 data_source="bloomberg",
                                                 freq="daily", cut="TOK",
                                                 ticker="USDJPY")

    print(output)

    output = cm.convert_vendor_to_library_ticker(
        data_source="bloomberg", vendor_tickers="EURUSD CMPT Curncy")

    print(output)

    output = cm.convert_vendor_to_library_field(
        data_source="bloomberg", vendor_fields="PX_LAST")

    print(output)

    output = cm.convert_library_to_vendor_field(
        data_source="bloomberg", vendor_fields="close")

    print(output)

    print(DataConstants().use_cache_compression)



__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on a "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


if __name__ == "__main__":
    ###### below line CRUCIAL when running Windows, otherwise multiprocessing 
    # doesn"t work! (not necessary on Linux)
    from findatapy.util import SwimPool;

    SwimPool()

    from findatapy.market import Market, MarketDataRequest, MarketDataGenerator

    market = Market(market_data_generator=MarketDataGenerator())

    # choose run_example = 0 for everything
    # run_example = 1 - download free tick data from DukasCopy example
    # run_example = 2 - download free FX daily data from Quandl
    # run_example = 3 - download FX data from FRED
    # run_example = 4 - download FX data from Bloomberg
    # run_example = 5 - download second FX data from Bloomberg
    # run_example = 6 - download free tick data from FXCM example
    # (compare with DukasCopy)

    run_example = 4

    if run_example == 1 or run_example == 0:
        ####### DukasCopy examples
        # let"s download data for 14 Jun 2016 for EUR/USD - the raw data has
        # bid/ask, if we specify close, we calculate
        # it as the average

        # first we can do it by defining all the vendor fields, tickers etc.
        # so we bypass the configuration file
        md_request = MarketDataRequest(start_date="14 Jun 2016",
                                       finish_date="15 Jun 2016",
                                       fields=["bid"], vendor_fields=["bid"],
                                       freq="tick", data_source="dukascopy",
                                       tickers=["EURUSD"],
                                       vendor_tickers=["EURUSD"])

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

        # now let"s do it using the category keyword, which goes into our
        # config files (only works for predefined tickers!)
        # simplifies our calling procedure a lot!
        md_request = MarketDataRequest(start_date="14 Jun 2016",
                                       finish_date="15 Jun 2016",
                                       category="fx", fields=["close"],
                                       freq="tick", data_source="dukascopy",
                                       tickers=["EURUSD"])

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

        # we can also get the bid/ask quotes directly
        md_request = MarketDataRequest(start_date="14 Jun 2016",
                                       finish_date="15 Jun 2016",
                                       category="fx", fields=["bid", "ask"],
                                       freq="tick", data_source="dukascopy",
                                       tickers=["EURUSD"])

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

    if run_example == 2 or run_example == 0:
        ####### Quandl data examples

        # for this to work make sure you edit the Quandl API key in
        # DataConstants file

        # already defined for us are the tickers for G10 USD FX in various
        # CSV files in conf folder
        # by default, assume data is for "close" field only, cut as being
        # "NYC" etc.
        # we can use keywords, "month" and "year" to specify the last year of
        # data, alternatively, we can put a specific date
        md_request = MarketDataRequest(start_date="year", category="fx",
                                       data_source="quandl",
                                       tickers=["EURUSD"])

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

        # if we give it a cross which doesn"t exist in the database directly,
        # it will try to synthesis from the USD rates
        # this might be ok for daily data, but should not be used for tick data
        # eg. AUD/JPY
        md_request = MarketDataRequest(start_date="year", category="fx",
                                       data_source="quandl",
                                       tickers=["AUDJPY"])

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

        # if you don"t specify tickers, it will download all the tickers in
        # that data_source.freq.category.cut combination
        # ie. quandl.daily.fx.NYC (all G10 USD crosses)
        md_request = MarketDataRequest(start_date="month", category="fx",
                                       data_source="quandl", cut="NYC")

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

    if run_example == 3 or run_example == 0:
        ####### FRED example

        # if we give it a cross which doesn"t exist in the database directly,
        # it will try to synthesis from the USD rates
        # this might be ok for daily data, but should not be used for tick data
        # eg. AUD/JPY
        md_request = MarketDataRequest(start_date="year", category="fx",
                                       data_source="quandl",
                                       tickers=["AUDJPY"])

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

        # if you don"t specify tickers, it will download all the tickers in
        # that data_source.freq.category.cut combination
        # ie. quandl.daily.fx.NYC (all G10 USD crosses)
        md_request = MarketDataRequest(start_date="month", category="fx",
                                       data_source="quandl", cut="NYC")

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

    if run_example == 4 or run_example == 0:
        ####### Bloomberg

        # now we will try various examples with Bloomberg
        # only works if you have Bloomberg terminal installed and the
        # Python API!
        md_request = MarketDataRequest(start_date="week", category="fx",
                                       data_source="bloomberg",
                                       tickers=["AUDJPY"])
        df = market.fetch_market(md_request)

        print(df.tail(n=10))

        # let"s now try downloading 1 minute intraday data for the past week
        # from Bloomberg
        md_request = MarketDataRequest(start_date="week", freq="intraday",
                                       category="fx", data_source="bloomberg",
                                       tickers=["AUDJPY"])

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

        # let"s now try downloading 1 minute intraday data for the past week
        # from Bloomberg, directly specifying tickers
        # bypassing our config file
        # this is handy when you want to download something which isn't
        # predefined in the configuration files
        md_request = MarketDataRequest(start_date="week", freq="intraday",
                                       data_source="bloomberg",
                                       tickers=["AUDJPY"],
                                       vendor_tickers=["AUDJPY BGN Curncy"],
                                       fields=["close"],
                                       vendor_fields=["close"])

        df = market.fetch_market(md_request)
        print(df.tail(n=10))

    if run_example == 5 or run_example == 0:
        ####### Bloomberg

        # let"s now try downloading tick data for the past hour from Bloomberg,
        # directly specifying tickers
        # bypassing our config file
        # this is handy when you want to download something which isn't
        # predefined in the configuration files
        # then convert into seconds data using pandas
        md_request = MarketDataRequest(start_date="hour", freq="tick",
                                       data_source="bloomberg",
                                       tickers=["AUDJPY"],
                                       vendor_tickers=["AUDJPY BGN Curncy"],
                                       fields=["close"],
                                       vendor_fields=["close"])

        df = market.fetch_market(md_request)
        df = df.resample("1s").mean()

        print(df.tail(n=60))

    if run_example == 6 or run_example == 0:
        ####### FXCM (and compare with DukasCopy) examples

        # let"s download data for end of 2016/start 2017 for EUR/USD - the
        # raw data has bid/ask, if we specify close, we calculate
        # it as the average

        # first we can do it by defining all the vendor fields, tickers etc.
        # so we bypass the configuration file
        md_request = MarketDataRequest(start_date="01 Dec 2016",
                                       finish_date="07 Dec 2016",
                                       fields=["bid"], vendor_fields=["bid"],
                                       freq="tick", data_source="fxcm",
                                       tickers=["EURUSD"],
                                       vendor_tickers=["EURUSD"])

        df_tick = market.fetch_market(md_request)
        df = df_tick.resample("1min").mean()

        md_request.data_source = "dukascopy"
        df1 = market.fetch_market(md_request).resample("1min").mean()

        df1.columns = [c + "_dk" for c in df1.columns]
        df = df1.join(df)

        print(df_tick.tail(n=100))
        print(df.tail(n=100))



__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2022 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on a "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#

if __name__ == "__main__":
    ###### below line CRUCIAL when running Windows, otherwise multiprocessing
    # doesn"t work! (not necessary on Linux)
    from findatapy.util import SwimPool;

    SwimPool()

    from findatapy.timeseries import Filter, Calendar, Calculations

    import pandas as pd

    calculations = Calculations()
    calendar = Calendar()
    filter = Filter()

    # choose run_example = 0 for everything
    # run_example = 1 - combine intraday dataframe with daily data dataframe

    run_example = 0

    if run_example == 1 or run_example == 0:
        df_intraday = pd.DataFrame(
            index=pd.date_range(start="01 Jan 2020", end="10 Jan 2020",
                                freq="1min"),
            columns=["ones"])
        df_intraday["ones"] = 1

        df_intraday = df_intraday.tz_localize("utc")

        df_daily = pd.DataFrame(
            index=pd.date_range(start="01 Jan 2020", end="10 Jan 2020",
                                freq="B"),
            columns=["ones", "twos"])

        df_daily["ones"] = 1
        df_daily["twos"] = 2

        df_joined = calculations.join_intraday_daily(
            df_intraday, df_daily, daily_time_of_day="10:00",
            daily_time_zone="Europe/London")

        print(df_joined)



__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on a "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


if __name__ == "__main__":
    ###### below line CRUCIAL when running Windows, otherwise multiprocessing
    # doesn"t work! (not necessary on Linux)
    from findatapy.util import SwimPool;

    SwimPool()

    from findatapy.market import Market, MarketDataRequest, MarketDataGenerator

    market = Market(market_data_generator=MarketDataGenerator())

    md_request = MarketDataRequest(
        start_date="01 Jun 2000",
        # start date (download data over past decade)
        data_source="alfred",  # use ALFRED/FRED as data source
        tickers=["US CPI YoY", "EZ CPI YoY"],  # ticker
        fields=["close"],  # which fields to download
        vendor_tickers=["CPIAUCSL", "CP0000EZ17M086NEST"],  # ticker (FRED)
        vendor_fields=["close"])  # which ALFRED fields to download

    df = market.fetch_market(md_request)

    print(df.tail(n=10))



__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on a "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#

if __name__ == "__main__":
    ###### below line CRUCIAL when running Windows, otherwise multiprocessing doesn"t work! (not necessary on Linux)
    from findatapy.util import SwimPool;

    SwimPool()

    from findatapy.market import Market, MarketDataRequest, MarketDataGenerator

    market = Market(market_data_generator=MarketDataGenerator())

    # choose run_example = 0 for everything
    # run_example = 1 - read in CSV file (daily)
    # run_example = 2 - read in HD5 file in market data (intraday)
    # run_example = 3 - save to disk for Quandl as Parquet (defined tickers)
    # and read back with MarketDataRequest
    # run_example = 4 - save to disk for Dukascopy as Parquet (defined tickers
    # for EURUSD) and read back with MarketDataRequest

    # NOTE: you need to make sure you have the correct data licences before
    # storing data on disk (and whether other
    # users can access it)

    run_example = 4

    if run_example == 1 or run_example == 0:
        md_request = MarketDataRequest(
            start_date="01 Jan 2002", finish_date="31 Jan 2016",
            tickers="S&P500",
            fields="close",
            data_source="../tests/S&P500.csv",
            freq="daily",
        )

        market = Market(market_data_generator=MarketDataGenerator())

        df = market.fetch_market(md_request=md_request)

        print(df)

    if run_example == 2 or run_example == 0:
        # load tick data from DukasCopy and then resample to 15s buckets
        md_request = MarketDataRequest(
            start_date="01 Jun 2016", finish_date="31 Jul 2016",
            tickers="EURUSD",
            fields="bid",
            data_source="../tests/EURUSD_tick.h5",
            freq="intraday",
            resample="15s",
            resample_how="last_dropna"
        )

        market = Market(market_data_generator=MarketDataGenerator())

        df = market.fetch_market(md_request=md_request)

        print(df)

        # the second time we call it, if we have Redis installed, we will fetch
        # from memory, so it will be quicker
        # also don"t need to run the resample operation again

        # need to specify cache_algo_return
        md_request.cache_algo = "cache_algo_return"

        df = market.fetch_market(md_request)

        print(df)

    if run_example == 3:
        # In this case we are saving predefined daily tickers to disk, and then
        # reading back
        from findatapy.util.dataconstants import DataConstants
        from findatapy.market.ioengine import IOEngine
        import os

        quandl_api_key = DataConstants().quandl_api_key  # change with your own
        # Quandl API key!

        md_request = MarketDataRequest(
            category="fx",
            data_source="quandl",
            freq="daily",
            quandl_api_key=quandl_api_key
        )

        market = Market(market_data_generator=MarketDataGenerator())

        df = market.fetch_market(md_request=md_request)

        print(df)

        folder = "../tests/"

        # Save to disk in a file name format friendly for reading later via
        # MarketDataRequest (ie. ../tests/backtest.fx.daily.quandl.NYC.parquet)
        IOEngine().write_time_series_cache_to_disk(folder, df,
                                                   engine="parquet",
                                                   md_request=md_request)

        md_request.data_engine = "../tests/*.parquet"

        df = market.fetch_market(md_request)

        print(df)

    if run_example == 4:
        # In this case we are saving predefined tick data tickers to disk, and
        # then reading back using the MarketDataRequest interface
        from findatapy.util.dataconstants import DataConstants
        from findatapy.market.ioengine import IOEngine

        md_request = MarketDataRequest(
            start_date="01 Jan 2021",
            finish_date="05 Jan 2021",
            category="fx",
            data_source="dukascopy",
            freq="tick",
            tickers=["EURUSD"],
            fields=["bid", "ask", "bidv", "askv"],
        )

        market = Market(market_data_generator=MarketDataGenerator())

        df = market.fetch_market(md_request=md_request)

        print(df)

        folder = "../tests/"

        # Save to disk in a format friendly for reading later
        # (ie. ../tests/backtest.fx.tick.dukascopy.NYC.EURUSD.parquet)
        # Here it will automatically generate the filename from the folder we
        # gave and the MarketDataRequest we made (altenatively, we could have
        # just given the filename directly)
        IOEngine().write_time_series_cache_to_disk(folder, df,
                                                   engine="parquet",
                                                   md_request=md_request)

        md_request.data_engine = "../tests/*.parquet"
        df = market.fetch_market(md_request)

        print(df)

        # Let's try the same thing with CSV too! But this will be slower given
        # it's CSV format...!
        IOEngine().write_time_series_cache_to_disk(folder, df, engine="csv",
                                                   md_request=md_request)

        md_request.data_engine = "../tests/*.csv"
        df = market.fetch_market(md_request)

        print(df)



__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on a "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#

if __name__ == "__main__":
    ###### below line CRUCIAL when running Windows, otherwise multiprocessing 
    # doesn"t work! (not necessary on Linux)
    from findatapy.util import SwimPool;

    SwimPool()

    from findatapy.timeseries import Filter, Calendar

    import pandas as pd

    calendar = Calendar()
    filter = Filter()

    # choose run_example = 0 for everything
    # run_example = 1 - get holidays for FX, EUR and EURUSD, as well as 
    #   listing weekends
    # run_example = 2 - get FX delivery dates and FX option expiries for 
    #   various tenors
    # run_example = 3 - get number of days between pandas DatetimeIndex
    # run_example = 4 - filter time series by EURUSD holidays
    # run_example = 4 - option expiries for USDJPY

    run_example = 0

    if run_example == 1 or run_example == 0:
        # Get the holidays (which aren"t weekends)
        print(calendar.get_holidays(start_date="01 Jan 1999 00:50",
                                    end_date="31 Dec 1999", cal="FX"))
        print(calendar.get_holidays(start_date="01 Jan 2000 00:10",
                                    end_date="31 Dec 2000", cal="EUR"))
        print(calendar.get_holidays(start_date="01 Jan 2000 00:10",
                                    end_date="31 Dec 2000", cal="EURUSD"))

        # Get the holidays (which are weekends)
        print(calendar.get_holidays(start_date="01 Jan 1999 00:50",
                                    end_date="31 Dec 1999", cal="WKD"))

    if run_example == 2 or run_example == 0:
        # Get delivery dates for these horizon dates - typically would use 
        # to get forward maturities
        print(calendar.get_delivery_date_from_horizon_date(
            pd.to_datetime([pd.Timestamp("02 Nov 2020")]), "ON", cal="EURUSD"))

        print(calendar.get_delivery_date_from_horizon_date(
            pd.to_datetime([pd.Timestamp("02 Nov 2020")]), "1W", cal="EURUSD"))

        print(calendar.get_delivery_date_from_horizon_date(
            pd.to_datetime([pd.Timestamp("27 Nov 2020")]), "1W", cal="EURUSD"))

        # Get 1M expires for these horizon dates - typically would use to get 
        # option expiries
        print(calendar.get_expiry_date_from_horizon_date(
            pd.to_datetime([pd.Timestamp("26 Oct 2020")]), "1M", cal="EURUSD"))

    if run_example == 3 or run_example == 0:
        # Create a list of business days and one which is + 1 day

        bus_days = pd.bdate_range("1 Jan 2020", "30 Jan 2020")
        bus_days_plus = bus_days + pd.Timedelta(days=7)

        print(calendar.get_delta_between_dates(bus_days, bus_days_plus))

    if run_example == 4 or run_example == 0:
        # Filter a time series by EURUSD holidays

        df = pd.DataFrame(index=pd.bdate_range("1 Jan 2020", "31 Dec 2020"))
        df["Prices"] = 1

        df_filtered = filter.filter_time_series_by_holidays(df, "EURUSD")

        print(len(df.index))
        print(len(df_filtered.index))

    if run_example == 5 or run_example == 0:
        # Get expiry for USDJPY

        # Get 1M expires for these horizon dates - typically would use to get
        # option expiries
        print(calendar.get_expiry_date_from_horizon_date(
            pd.to_datetime([pd.Timestamp("26 Nov 2008")]), "1M", cal="USDJPY"))



__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on a "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


from findatapy.market import MarketDataGenerator, Market, MarketDataRequest


def generate_market_data_for_tests():
    # generate daily S&P500 data from Quandl
    md_request = MarketDataRequest(start_date="01 Jan 2001",
                                   finish_date="01 Dec 2008",
                                   tickers=["S&P500"],
                                   vendor_tickers=["YAHOO/INDEX_GSPC"],
                                   fields=["close"],
                                   data_source="quandl")

    market = Market(market_data_generator=MarketDataGenerator())

    df = market.fetch_market(md_request)
    df.to_csv("S&P500.csv")

    # generate tick data from DukasCopy for EURUSD
    md_request = MarketDataRequest(start_date="14 Jun 2016",
                                   finish_date="15 Jun 2016", cut="NYC",
                                   category="fx",
                                   fields=["bid"], freq="tick",
                                   data_source="dukascopy",
                                   tickers=["EURUSD"])

    market = Market(market_data_generator=MarketDataGenerator())

    df = market.fetch_market(md_request)
    df.to_csv("EURUSD_tick.csv")


if __name__ == "__main__":
    generate_market_data_for_tests()



__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on a "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


# TODO: Needs fixing, given change in APIs

if __name__ == "__main__":
    ###### below line CRUCIAL when running Windows, otherwise multiprocessing 
    # doesn"t work! (not necessary on Linux)
    from findatapy.util import SwimPool;

    SwimPool()

    from findatapy.market import Market, MarketDataRequest, MarketDataGenerator

    market = Market(market_data_generator=MarketDataGenerator())

    # Choose run_example   (0 = all examples)
    # Example 1: bitcoincharts
    # Example 2: poloniex (needs to be fixed)
    # Example 3: binance
    # Example 4: bitfinex
    # Example 5: gdax
    # Example 6: kraken
    #
    # Check findatapy/conf/time_series_tickers_list.csv for all possible tickers
    # Note we use XBT instead of BTC.  Same for XET (ETH) and XLC (LTC).
    #
    # Note: there will generally be a limit of how many data points you can 
    # pull in one call, so you might
    # have to choose smaller time periods

    run_example = 0

    if run_example == 1 or run_example == 0:
        ### Download data from bitcoincharts ###
        # fields contains ["close","volume"]
        # return tick data

        md_request = MarketDataRequest(start_date="11 Nov 2015",
                                       finish_date="02 Feb 2018", cut="LOC",
                                       freq="tick",
                                       data_source="bitcoincharts",
                                       category="crypto",
                                       fields=["close", "volume"],
                                       tickers=["XBTUSD_itbit"])

        df = market.fetch_market(md_request)
        print(df.head(5))
        print(df.tail(5))

    if run_example == 2 or run_example == 0:
        ### Download data from poloniex ###
        # freq : daily or intraday, where intraday - returns 5 minutes data.
        # fields contains ["close","high","low","open","quote-volume","volume","weighted-average"]

        md_request = MarketDataRequest(start_date="18 Feb 2017",
                                       finish_date="20 Feb 2017", cut="LOC",
                                       freq="intraday", data_source="poloniex",
                                       category="crypto",
                                       fields=["close", "volume",
                                               "weighted-average"],
                                       tickers=["STRXBT"])

        df = market.fetch_market(md_request)
        print(df.head(5))
        print(df.tail(5))

    if run_example == 3 or run_example == 0:
        ### download data from binance ###
        # freq: daily or intraday.  intraday - return 1 minute data
        # fields contain ["open","high","low","close","volume","quote-asset-volume",
        #                 "trade-numbers","taker-buy-base-asset-volume","taker-buy-quote-asset-volume"]
        # Note it may take a while due to the limit of numbers per second calling the API

        md_request = MarketDataRequest(start_date="01 Feb 2017",
                                       finish_date="20 Feb 2018", cut="LOC",
                                       freq="daily", data_source="binance",
                                       category="crypto",
                                       fields=["close", "volume",
                                               "quote-asset-volume"],
                                       tickers=["WTCXBT"])

        df = market.fetch_market(md_request)
        print(df.head(5))
        print(df.tail(5))

    if run_example == 4 or run_example == 0:
        ### download data from bitfinex ###
        # freq: daily or intraday.  intraday - return 1 minute data
        # fields contain ["open","close","high","low","volume"]
        # Note it may take a while due to the limit of numbers per second calling the API

        md_request = MarketDataRequest(start_date="11 Feb 2018",
                                       finish_date="20 Feb 2018", cut="LOC",
                                       freq="intraday", data_source="bitfinex",
                                       category="crypto",
                                       fields=["close", "volume", "high",
                                               "open"],
                                       tickers=["XLCUSD"])

        df = market.fetch_market(md_request)
        print(df.head(5))
        print(df.tail(5))

    if run_example == 5 or run_example == 0:
        ### Download data from gdax (now called coinbase pro) ###
        # freq: daily or intraday.  intraday - 1 minute data
        # fields contain ["low","high","open","close","volume"]
        # Note it may take a while due to the limit of numbers per second calling the API

        md_request = MarketDataRequest(start_date="01 Jan 2018 00:00",
                                       finish_date="01 Jan 2018 01:50",
                                       cut="LOC",
                                       freq="intraday", data_source="gdax",
                                       category="crypto",
                                       fields=["close", "volume", "low",
                                               "high"],
                                       tickers=["XBTUSD"])

        df = market.fetch_market(md_request)
        print(df.head(5))
        print(df.tail(5))

    if run_example == 6:
        ### Download data from kraken ###
        # freq: tick (all trade data)
        # fields contain ["close","volume","buy-sell","market-limit"]
        # buy-sell : 1 means buy, and -1 means sell
        # market-limit: 1 means market, and -1 means limit

        # Note it may take a while due to the limit of numbers per second for
        # calling the API

        md_request = MarketDataRequest(start_date="19 Feb 2018",
                                       finish_date="20 Feb 2018", cut="LOC",
                                       freq="tick", data_source="kraken",
                                       category="crypto",
                                       fields=["close", "volume", "buy-sell",
                                               "market-limit"],
                                       tickers=["XBTUSD"])

        df = market.fetch_market(md_request)
        print(df.head(5))
        print(df.tail(5))
